# Copyright 2015-2018 Klarna Bank AB
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import unicode_literals
import traceback
import logging
from django.shortcuts import render
from django.views.generic import FormView, DetailView
from django.views.generic.base import TemplateView
from django.core.exceptions import PermissionDenied
from django.utils.decorators import method_decorator
from django.utils.safestring import mark_safe
from django.conf import settings
from rest_framework.views import APIView
from rest_framework.generics import GenericAPIView
from rest_framework.response import Response
from rest_framework import status
from rest_framework import permissions
from django_rethink import *
from socrates_api.tasks import *
from socrates_api.serializers import *

logger = logging.getLogger("socrates.gui.views")

class AssetConsolePermission(permissions.BasePermission):
    def has_object_permission(self, request, view, obj):
        return (view.get_serializer(obj).has_write_permission(request.user) and
                (request.user.is_superuser or request.user.is_console_user or
                 obj['asset_type'] == 'vm'))

class AssetConsoleView(RethinkAPIMixin, GenericAPIView):
    table_name = 'assets'
    slug_index_name = 'service_tag'
    group_filter_fields = ['owner', 'managers']
    serializer_class = AssetSerializer
    permission_classes = (permissions.IsAuthenticated, AssetConsolePermission)
    def get(self, request, format=None, slug=None):
        context = {'object': self.get_object()}
        if context['object']['asset_type'] == 'server':
            context['hostname'] = "%s.%s" % (context['object']['service_tag'], settings.SOCRATES_OOB_DOMAIN)
            context['is_idrac6'] = any([context['object']['model'].startswith(model) for model in ["PowerEdge R310", "PowerEdge R610", "PowerEdge R710", "PowerEdge R910"]])
            response = render(request, 'asset_console.jnlp', context=context, content_type='application/x-java-jnlp-file')
            response['Content-Disposition'] = 'attachment; filename="viewer.jnlp"'
            return response
        elif context['object']['asset_type'] == 'vm' and context['object']['asset_subtype'] == 'libvirt':
            return render(request, 'asset_console_spice.html', context=context)
        else:
            return Response(status=status.HTTP_501_NOT_IMPLEMENTED)

class AssetManagementRedirectView(RethinkAPIMixin, GenericAPIView):
    table_name = 'assets'
    slug_index_name = 'service_tag'
    group_filter_fields = ['owner', 'managers']
    serializer_class = AssetSerializer
    permission_classes = (permissions.IsAuthenticated, AssetConsolePermission)
    def get(self, request, format=None, slug=None):
        return Response(status=status.HTTP_302_FOUND, headers={'Location': 'https://' + slug + '.' + settings.SOCRATES_OOB_DOMAIN})

class ReactView(TemplateView):
    template_name = 'app.html'

class ReQLView(RethinkAPIMixin, APIView):
    permission_classes = (permissions.IsAuthenticated, permissions.IsAdminUser)
    def post(self, request, format=None):
        try:
            query = eval(request.data["query"], {'r': r})
        except:
            return Response({'query': [traceback.format_exc()]}, status=status.HTTP_400_BAD_REQUEST)
        if not isinstance(query, r.RqlQuery):
            return Response({'query': ["query didn't result in a RqlQuery object: %s" % type(query)]}, status=status.HTTP_400_BAD_REQUEST)
        logger.warning("%s executed %s" % (request.user.username, request.data["query"]))
        try:
            result = query.run(get_connection())
            if isinstance(result, r.errors.ReqlCursorEmpty):
                result = list(result)
        except:
            return Response({'query': [traceback.format_exc()]}, status=status.HTTP_400_BAD_REQUEST)
        return Response(result, status=status.HTTP_200_OK)



# Copyright 2015-2018 Klarna Bank AB
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

STATICFILES_STORAGE = 'pipeline.storage.PipelineCachedStorage'

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'pipeline.finders.PipelineFinder',
)

STATICFILES_DIRS = [
    os.path.join(os.path.dirname(__file__), 'node_modules', 'bootstrap', 'dist'),
]

PIPELINE_COMPILERS = (
    'pipeline_browserify.compiler.BrowserifyCompiler',
)

PIPELINE_CSS_COMPRESSOR = 'pipeline.compressors.NoopCompressor'
PIPELINE_JS_COMPRESSOR = 'pipeline.compressors.uglifyjs.UglifyJSCompressor'

PIPELINE_BROWSERIFY_ARGUMENTS = '-t [ babelify --presets [ es2015 react ] ]'

PIPELINE_CSS = {
    'socrates_css': {
        'source_filenames': (
            'css/style.css',
        ),
        'output_filename': 'css/socrates.css',
    },
}

PIPELINE_JS = {
    'socrates_js': {
        'source_filenames': (
            'node_modules/jquery/dist/jquery.min.js',
            'node_modules/bootstrap/dist/js/bootstrap.min.js',
            'node_modules/react/dist/react-with-addons.min.js',
            'js/app.socrates.browserify.js',
        ),
        'output_filename': 'js/socrates.js',
    },
    'spice_html5_js': {
        'source_filenames': (
            'js/app.spice-html5.browserify.js',
        ),
        'output_filename': 'js/spice-html5.js',
    },
}

PIPELINE = {
    'JAVASCRIPT': PIPELINE_JS,
    'STYLESHEETS': PIPELINE_CSS,
    'COMPILERS': PIPELINE_COMPILERS,
    'BROWSERIFY_BINARY': 'browserify',
    'BROWSERIFY_ARGUMENTS': PIPELINE_BROWSERIFY_ARGUMENTS,
    'JS_COMPRESSOR': PIPELINE_JS_COMPRESSOR,
    'CSS_COMPRESSOR': PIPELINE_CSS_COMPRESSOR,
}



# Copyright 2018 Klarna Bank AB
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf import settings
from django.core import management
from django.test import override_settings
from django.urls import reverse
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from selenium.webdriver.firefox.webdriver import WebDriver
import rethinkdb as r

class SeleniumTests(StaticLiveServerTestCase):
    @classmethod
    def setUpClass(cls):
        super(SeleniumTests, cls).setUpClass()
        cls.selenium = WebDriver()
        cls.selenium.implicitly_wait(10)
        cls.conn = r.connect(host=settings.RETHINK_DB_HOST, port=settings.RETHINK_DB_PORT)
        try:
            r.db_drop(settings.RETHINK_DB_DB).run(cls.conn)
        except:
            pass
        r.db_create(settings.RETHINK_DB_DB).run(cls.conn)
        cls.conn.db = settings.RETHINK_DB_DB
        management.call_command('syncrethinkdb', verbosity=0)

    @classmethod
    def tearDownClass(cls):
        cls.selenium.quit()
        super(SeleniumTests, cls).tearDownClass()

    def test_page_load(self):
        self.selenium.get('%s%s' % (self.live_server_url, reverse('socrates_gui:home_view')))
        app = self.selenium.find_element_by_id("socrates_app")



# Copyright 2015-2018 Klarna Bank AB
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import random
import os
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "socrates.settings")
import django
django.setup()
from django.core import management
management.call_command('migrate', verbosity=0)
from django.contrib.auth import authenticate
from socrates_api.serializers import AssetSerializer
from socrates_api.tasks import *
from websockify.auth_plugins import AuthenticationError

class SocratesHost(str):
    def __new__(cls, host, asset):
        obj = str.__new__(cls, host)
        obj.asset = asset
        return obj

class SocratesTokenPlugin(object):
    def __init__(self, src=None):
        pass
    def lookup(self, token):
        asset = AssetSerializer.get(id=token)
        if asset['asset_type'] == "vm" and asset['asset_subtype'] == "libvirt":
            parent_asset = AssetSerializer.get(service_tag=asset['parent'])
            api = connect_hypervisor_libvirt(parent_asset)
            vm = find_vm_libvirt(api, asset)
            if vm.isActive():
                root = etree.fromstring(vm.XMLDesc())
                graphics = root.xpath("/domain/devices/graphics")[0]
                host = graphics.attrib['listen']
                port = graphics.attrib['port']
                if host == "127.0.0.1" and parent_asset['url'].startswith("qemu+ssh://"):
                    url = urlparse.urlparse(parent_asset['url'])
                    # FIXME: random sucks...
                    local_port = random.randint(5900, 7000)
                    subprocess.call(["ssh", "-f", "-T", "-L", "%d:%s:%s" % (local_port, host, port), url.netloc, "sleep", "10"])
                    port = local_port
            api.close()
        return SocratesHost(host, asset), port

class SocratesAuthPlugin(object):
    def __init__(self, src=None):
        self.src = src

    def authenticate(self, headers, target_host, target_port):
        import base64
        auth_header = headers.get('Authorization')
        if auth_header:
            if not auth_header.startswith('Basic '):
                raise AuthenticationError(response_code=403)

            try:
                user_pass_raw = base64.b64decode(auth_header[6:])
            except TypeError:
                raise AuthenticationError(response_code=403)

            try:
                # http://stackoverflow.com/questions/7242316/what-encoding-should-i-use-for-http-basic-authentication
                user_pass_as_text = user_pass_raw.decode('ISO-8859-1')
            except UnicodeDecodeError:
                raise AuthenticationError(response_code=403)

            user_pass = user_pass_as_text.split(':', 1)
            if len(user_pass) != 2:
                raise AuthenticationError(response_code=403)

            user = authenticate(username=user_pass[0], password=user_pass[1])
            if user is None:
                raise AuthenticationError(response_code=403)

            if not AssetSerializer(target_host.asset).has_write_permission(user):
                raise AuthenticationError(response_code=403)

        else:
            raise AuthenticationError(response_code=401,
                                      response_headers={'WWW-Authenticate': 'Basic realm="api"'})



#!/usr/bin/env python

from setuptools import setup
import os
import re

setup(name='socrates_gui',
      version='1.0.3',
      license='Apache Software License',
      description='Source of Truth for hardware, virtual machines, and networks',
      author='Klarna Bank AB',
      author_email='daniel.zakrisson@klarna.com',
      url='https://github.com/dhozac/socrates_gui',
      packages=['socrates_gui'],
      install_requires=[
          'websockify',
          'socrates_api',
          'django-pipeline',
          'django-pipeline-browserify',
      ],
      include_package_data=True,
      zip_safe=False,
      classifiers=[
          'Development Status :: 7 - Inactive',
          'Environment :: Web Environment',
          'Framework :: Django :: 1.11',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: Apache Software License',
          'Topic :: System :: Installation/Setup',
      ],
     )



# Copyright 2015-2018 Klarna Bank AB
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf.urls import include, url
from socrates_gui.views import *
urlpatterns = [
    url(r'^asset/(?P<slug>[A-Za-z0-9 -]+?)/console/?$', AssetConsoleView.as_view(), name='asset_console'),
    url(r'^asset/(?P<slug>[A-Za-z0-9 -]+?)/management/?$', AssetManagementRedirectView.as_view(), name='asset_mgmt_redirect'),
    url(r'^reqlapi/$', ReQLView.as_view(), name='reql_view'),
    url(r'^.*', ReactView.as_view(), name='home_view'),
]
